@@ -20,17 +20,17 @@
 
 #include <net-snmp/net-snmp-config.h>
 
-#if HAVE_STDLIB_H
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if TIME_WITH_SYS_TIME
+#ifdef TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
 #else
-# if HAVE_SYS_TIME_H
+# ifdef HAVE_SYS_TIME_H
 #  include <sys/time.h>
 # else
 #  include <time.h>
@@ -129,6 +129,8 @@
          0;
 #endif
 
+static unsigned char zero_octet_string[1];
+
 static int
 fetch_var_val(oid * name, size_t namelen, u_long * new_value)
 {
@@ -645,7 +647,7 @@ var_alarmEntry(struct variable * vp, oid * name, size_t * length,
             return (unsigned char *) hdr->owner;
         } else {
             *var_len = 0;
-            return (unsigned char *) "";
+            return zero_octet_string;
         }
 
     case IDalarmStatus:
@@ -661,7 +663,7 @@ var_alarmEntry(struct variable * vp, oid * name, size_t * length,
 }
 
 /*
- * Registration & Initializatio section 
+ * Registration & Initialization section
  */
 
 oid             oidalarmVariablesOid[] = { 1, 3, 6, 1, 2, 1, 16, 3 };
