@@ -1,7 +1,7 @@
 /*
  * systemstats data access header
  *
- * $Id: systemstats.h,v 1.2 2005/10/28 15:29:23 rstory Exp $
+ * $Id$
  */
 /**---------------------------------------------------------------------*/
 /*
@@ -17,14 +17,13 @@
  *    HPUX11 had different reuirements than other HPUX, that should
  *    be handled in the *_hpux.h header file.
  */
-config_require(ip-mib/data_access/systemstats_common)
+config_require(ip-mib/data_access/systemstats_common);
 #if defined( linux )
-config_require(ip-mib/data_access/systemstats_linux)
+config_require(ip-mib/data_access/systemstats_linux);
+#elif defined( solaris2 )
+config_require(ip-mib/data_access/systemstats_solaris2);
+#elif defined( freebsd7 ) || defined( netbsd5 ) || defined( openbsd4 ) || defined( dragonfly ) || defined( darwin )
+config_require(ip-mib/data_access/systemstats_sysctl);
 #else
-/*
- * couldn't determine the correct file!
- * require a bogus file to generate an error.
- */
-config_require(ip-mib/data_access/systemstats-unknown-arch);
+config_error(the systemstats data access library is not available in this environment.);
 #endif
-
