@@ -26,10 +26,10 @@
 #include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#if HAVE_UNISTD_H
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
@@ -38,10 +38,6 @@
 #include <netinet/in.h>
 #endif
 
-#if HAVE_WINSOCK_H
-#include <winsock.h>
-#endif
-
 #include <net-snmp/net-snmp-includes.h>
 
 #include <stdlib.h>
@@ -87,7 +83,7 @@ char           *newpass = NULL, *oldpass = NULL;
 char           *transform_type_input = NULL;
 
 const oid      *transform_type = NULL;  /* Type of HMAC hash to use.      */
-
+size_t          transform_type_len = 0;
 
 
 /*
@@ -99,12 +95,12 @@ int             get_user_passphrases(void);
 int             snmp_ttyecho(const int fd, const int echo);
 char           *snmp_getpassphrase(const char *prompt, int fvisible);
 
-#ifdef WIN32
+#if defined(HAVE__CPUTS) && defined(HAVE__GETCH)
+#include <conio.h>
+#include <io.h>
 #define HAVE_GETPASS 1
-char           *getpass(const char *prompt);
-int             isatty(int);
-int             _cputs(const char *);
-int             _getch(void);
+#define isatty _isatty
+static char    *getpass(const char *prompt);
 #endif
 
 /*******************************************************************-o-******
@@ -112,7 +108,7 @@ int             _getch(void);
 int
 main(int argc, char **argv)
 {
-    int             rval = SNMPERR_SUCCESS;
+    int             rval = 1;
     size_t          oldKu_len = SNMP_MAXBUF_SMALL,
         newKu_len = SNMP_MAXBUF_SMALL,
         oldkul_len = SNMP_MAXBUF_SMALL,
@@ -124,7 +120,7 @@ main(int argc, char **argv)
         oldkul[SNMP_MAXBUF_SMALL],
         newkul[SNMP_MAXBUF_SMALL], keychange[SNMP_MAXBUF_SMALL];
 
-    int             i;
+    int             i, auth_type;
     int             arg = 1;
 
     local_progname = argv[0];
@@ -135,8 +131,9 @@ main(int argc, char **argv)
         fprintf(stderr, "%s: out of memory!", local_progname);
         exit(-1);
     }
-    sprintf(local_passphrase_filename, "%s/%s", PASSPHRASE_DIR,
-            PASSPHRASE_FILE);
+    snprintf(local_passphrase_filename, sizeof(PASSPHRASE_DIR) +
+                                        sizeof(PASSPHRASE_FILE) + 4,
+             "%s/%s", PASSPHRASE_DIR, PASSPHRASE_FILE);
 
 
 
@@ -174,6 +171,7 @@ main(int argc, char **argv)
             break;
         case 'h':
             rval = 0;
+	    /* fallthrough */
         default:
             usage_to_file(stdout);
             exit(rval);
@@ -191,31 +189,18 @@ main(int argc, char **argv)
     /*
      * Convert and error check transform_type.
      */
-#ifndef NETSNMP_DISABLE_MD5
-    if (!strcmp(transform_type_input, "md5")) {
-        transform_type = usmHMACMD5AuthProtocol;
-
-    } else
-#endif
-        if (!strcmp(transform_type_input, "sha1")) {
-        transform_type = usmHMACSHA1AuthProtocol;
-
-    } else {
+    auth_type = usm_lookup_auth_type(transform_type_input);
+    transform_type = sc_get_auth_oid( auth_type, &transform_type_len );
+    if (NULL == transform_type) {
         fprintf(stderr,
                 "Unrecognized hash transform: \"%s\".\n",
                 transform_type_input);
         usage_synopsis(stderr);
-        QUITFUN(rval = SNMPERR_GENERR, main_quit);
+        QUITFUN(SNMPERR_GENERR, main_quit);
     }
 
     if (verbose) {
-        fprintf(stderr, "Hash:\t\t%s\n",
-#ifndef NETSNMP_DISABLE_MD5
-                (transform_type == usmHMACMD5AuthProtocol)
-                ? "usmHMACMD5AuthProtocol" :
-#endif
-                "usmHMACSHA1AuthProtocol"
-            );
+        fprintf(stderr, "Hash:\t\t%s\n", sc_get_auth_name(auth_type));
     }
 
 
@@ -236,7 +221,8 @@ main(int argc, char **argv)
                     (unsigned long)engineid_len));
     } else {
         engineid_len = setup_engineID(&engineid, (char *) engineid);
-
+        if ((ssize_t)engineid_len < 0)
+            exit(1);
     }
 
 #ifdef NETSNMP_ENABLE_TESTING_CODE
@@ -258,12 +244,12 @@ main(int argc, char **argv)
     if (strlen(oldpass) < USM_LENGTH_P_MIN) {
         fprintf(stderr, "Old passphrase must be greater than %d "
                 "characters in length.\n", USM_LENGTH_P_MIN);
-        QUITFUN(rval = SNMPERR_GENERR, main_quit);
+        QUITFUN(SNMPERR_GENERR, main_quit);
 
     } else if (strlen(newpass) < USM_LENGTH_P_MIN) {
         fprintf(stderr, "New passphrase must be greater than %d "
                 "characters in length.\n", USM_LENGTH_P_MIN);
-        QUITFUN(rval = SNMPERR_GENERR, main_quit);
+        QUITFUN(SNMPERR_GENERR, main_quit);
     }
 
     if (verbose) {
@@ -282,13 +268,13 @@ main(int argc, char **argv)
     QUITFUN(rval, main_quit);
 
 
-    rval = generate_Ku(transform_type, USM_LENGTH_OID_TRANSFORM,
+    rval = generate_Ku(transform_type, transform_type_len,
                        (u_char *) oldpass, strlen(oldpass),
                        oldKu, &oldKu_len);
     QUITFUN(rval, main_quit);
 
 
-    rval = generate_Ku(transform_type, USM_LENGTH_OID_TRANSFORM,
+    rval = generate_Ku(transform_type, transform_type_len,
                        (u_char *) newpass, strlen(newpass),
                        newKu, &newKu_len);
     QUITFUN(rval, main_quit);
@@ -305,7 +291,7 @@ main(int argc, char **argv)
         DEBUGMSGTL(("encode_keychange", "%02x", (int) (oldKu[i])));
     DEBUGMSGTL(("encode_keychange", "\n"));
 
-    rval = generate_kul(transform_type, USM_LENGTH_OID_TRANSFORM,
+    rval = generate_kul(transform_type, transform_type_len,
                         engineid, engineid_len,
                         oldKu, oldKu_len, oldkul, &oldkul_len);
     QUITFUN(rval, main_quit);
@@ -317,7 +303,7 @@ main(int argc, char **argv)
         DEBUGMSGTL(("encode_keychange", "%02x", (int) (oldkul[i])));
     DEBUGMSGTL(("encode_keychange", "\n"));
 
-    rval = generate_kul(transform_type, USM_LENGTH_OID_TRANSFORM,
+    rval = generate_kul(transform_type, transform_type_len,
                         engineid, engineid_len,
                         newKu, newKu_len, newkul, &newkul_len);
     QUITFUN(rval, main_quit);
@@ -328,7 +314,7 @@ main(int argc, char **argv)
         DEBUGMSGTL(("encode_keychange", "%02x", newkul[i]));
     DEBUGMSGTL(("encode_keychange", "\n"));
 
-    rval = encode_keychange(transform_type, USM_LENGTH_OID_TRANSFORM,
+    rval = encode_keychange(transform_type, transform_type_len,
                             oldkul, oldkul_len,
                             newkul, newkul_len, keychange, &keychange_len);
     QUITFUN(rval, main_quit);
@@ -351,11 +337,11 @@ main(int argc, char **argv)
     SNMP_ZERO(oldpass, strlen(oldpass));
     SNMP_ZERO(newpass, strlen(newpass));
 
-    SNMP_ZERO(oldKu, oldKu_len);
-    SNMP_ZERO(newKu, newKu_len);
+    memset(oldKu, 0, oldKu_len);
+    memset(newKu, 0, newKu_len);
 
-    SNMP_ZERO(oldkul, oldkul_len);
-    SNMP_ZERO(newkul, newkul_len);
+    memset(oldkul, 0, oldkul_len);
+    memset(newkul, 0, newkul_len);
 
     SNMP_ZERO(s, strlen(s));
 
@@ -389,26 +375,26 @@ usage_synopsis(FILE * ofp)
 void
 usage_to_file(FILE * ofp)
 {
-    char           *s;
+    const char *s;
 
     usage_synopsis(ofp);
 
-    fprintf(ofp, "\n\
-    Only -t is mandatory.  The transform is used to convert P=>Ku, convert\n\
-    Ku=>Kul, and to hash the old Kul with the random bits.\n\
-\n\
-    Passphrase will be taken from the first successful source as follows:\n\
+    fprintf(ofp, "\n%s\
 	a) Commandline options,\n\
 	b) The file \"%s/%s\",\n\
-	c) stdin  -or-  User input from the terminal.\n\
+	c) stdin  -or-  User input from the terminal.\n\n%s\
+		" NL,
+   "Only -t is mandatory.  The transform is used to convert P=>Ku, convert\n\
+    Ku=>Kul, and to hash the old Kul with the random bits.\n\
 \n\
-    -f will require reading from the stdin/terminal, ignoring a) and b).\n\
+    Passphrase will be taken from the first successful source as follows:\n",
+    (s = netsnmp_gethomedir()) ? s : "$HOME", local_passphrase_filename,
+   "-f will require reading from the stdin/terminal, ignoring a) and b).\n\
     -P will prevent prompts for passphrases to stdout from being printed.\n\
 \n\
-    <engineID> is interpreted as a hex string when preceeded by \"0x\",\n\
+    <engineID> is interpreted as a hex string when preceded by \"0x\",\n\
     otherwise it is created to contain \"text\".  If nothing is given,\n\
-    <engineID> is constructed from the first IP address for the local host.\n\
-		" NL, (s = getenv("HOME")) ? s : "$HOME", local_passphrase_filename);
+    <engineID> is constructed from the first IP address for the local host.\n");
 
 
     /*
@@ -475,7 +461,8 @@ get_user_passphrases(void)
 
     char           *obuf = NULL, *nbuf = NULL;
 
-    char            path[SNMP_MAXBUF], buf[SNMP_MAXBUF], *s = NULL;
+    char            path[SNMP_MAXBUF], buf[SNMP_MAXBUF];
+    const char     *s = NULL;
 
     struct stat     statbuf;
     FILE           *fp = NULL;
@@ -498,7 +485,7 @@ get_user_passphrases(void)
      * path for existence and access first.  Refuse to read
      * if the permissions are wrong.
      */
-    s = getenv("HOME");
+    s = netsnmp_gethomedir();
     snprintf(path, sizeof(path), "%s/%s", s, PASSPHRASE_DIR);
     path[ sizeof(path)-1 ] = 0;
 
@@ -507,13 +494,13 @@ get_user_passphrases(void)
      */
     if (stat(path, &statbuf) < 0) {
         fprintf(stderr, "Cannot access directory \"%s\".\n", path);
-        QUITFUN(rval = SNMPERR_GENERR, get_user_passphrases_quit);
+        QUITFUN(SNMPERR_GENERR, get_user_passphrases_quit);
 #ifndef WIN32
     } else if (statbuf.st_mode & (S_IRWXG | S_IRWXO)) {
         fprintf(stderr,
                 "Directory \"%s\" is accessible by group or world.\n",
                 path);
-        QUITFUN(rval = SNMPERR_GENERR, get_user_passphrases_quit);
+        QUITFUN(SNMPERR_GENERR, get_user_passphrases_quit);
 #endif                          /* !WIN32 */
     }
 
@@ -524,12 +511,12 @@ get_user_passphrases(void)
     path[ sizeof(path)-1 ] = 0;
     if (stat(path, &statbuf) < 0) {
         fprintf(stderr, "Cannot access file \"%s\".\n", path);
-        QUITFUN(rval = SNMPERR_GENERR, get_user_passphrases_quit);
+        QUITFUN(SNMPERR_GENERR, get_user_passphrases_quit);
 #ifndef WIN32
     } else if (statbuf.st_mode & (S_IRWXG | S_IRWXO)) {
         fprintf(stderr,
                 "File \"%s\" is accessible by group or world.\n", path);
-        QUITFUN(rval = SNMPERR_GENERR, get_user_passphrases_quit);
+        QUITFUN(SNMPERR_GENERR, get_user_passphrases_quit);
 #endif                          /* !WIN32 */
     }
 
@@ -538,7 +525,7 @@ get_user_passphrases(void)
      */
     if ((fp = fopen(path, "r")) == NULL) {
         fprintf(stderr, "Cannot open \"%s\".", path);
-        QUITFUN(rval = SNMPERR_GENERR, get_user_passphrases_quit);
+        QUITFUN(SNMPERR_GENERR, get_user_passphrases_quit);
     }
 
     /*
@@ -552,7 +539,7 @@ get_user_passphrases(void)
 
     } else if (!oldpass) {
         len = strlen(buf);
-        if (buf[len - 1] == '\n')
+        if (len && buf[len - 1] == '\n')
             buf[--len] = '\0';
         oldpass = (char *) calloc(1, len + 1);
         if (oldpass)
@@ -568,7 +555,7 @@ get_user_passphrases(void)
 
     } else if (!newpass) {
         len = strlen(buf);
-        if (buf[len - 1] == '\n')
+        if (len && buf[len - 1] == '\n')
             buf[--len] = '\0';
         newpass = (char *) calloc(1, len + 1);
         if (newpass)
@@ -613,7 +600,7 @@ get_user_passphrases(void)
 
 
   get_user_passphrases_quit:
-    SNMP_ZERO(buf, SNMP_MAXBUF);
+    memset(buf, 0, SNMP_MAXBUF);
 
     if (obuf != oldpass) {
         SNMP_ZERO(obuf, strlen(obuf));
@@ -743,34 +730,37 @@ snmp_getpassphrase(const char *prompt, int bvisible)
         ti = snmp_ttyecho(0, 0);
     }
 
-    fgets(buffer, sizeof(buffer), stdin);
+    bufp = fgets(buffer, sizeof(buffer), stdin);
 
     if (!bvisible) {
         ti = snmp_ttyecho(0, ti);
         fputs("\n", ofp);
     }
+    if (!bufp) {
+        fprintf(stderr, "Aborted...\n");
+        exit(1);
+    }
 
 
     /*
      * Copy the input and zero out the read-in buffer.
      */
     len = strlen(buffer);
-    if (buffer[len - 1] == '\n')
+    if (len && buffer[len - 1] == '\n')
         buffer[--len] = '\0';
 
     bufp = (char *) calloc(1, len + 1);
     if (bufp)
         memcpy(bufp, buffer, len + 1);
 
-    SNMP_ZERO(buffer, SNMP_MAXBUF);
+    memset(buffer, 0, SNMP_MAXBUF);
 
 
     return bufp;
 
 }                               /* end snmp_getpassphrase() */
 
-#ifdef WIN32
-
+#if defined(HAVE__CPUTS) && defined(HAVE__GETCH)
 int
 snmp_ttyecho(const int fd, const int echo)
 {
@@ -781,7 +771,7 @@ snmp_ttyecho(const int fd, const int echo)
  * stops at the first newline, carrier return, or backspace.
  * WARNING! _getch does NOT read <Ctrl-C>
  */
-char           *
+static char    *
 getpass(const char *prompt)
 {
     static char     pbuf[128];
@@ -799,4 +789,4 @@ getpass(const char *prompt)
 
     return pbuf;
 }
-#endif                          /* WIN32 */
+#endif /* defined(HAVE__CPUTS) && defined(HAVE__GETCH) */
