@@ -6,10 +6,50 @@
 #include <net-snmp/net-snmp-includes.h>
 #include <net-snmp/library/snmp_transport.h>
 
+static char**
+create_word_array_helper(const char* cptr, size_t idx, char* tmp, size_t tmplen)
+{
+    char* item;
+    char** res;
+    cptr = copy_nword_const(cptr, tmp, tmplen);
+    item = strdup(tmp);
+    if (cptr)
+        res = create_word_array_helper(cptr, idx + 1, tmp, tmplen);
+    else {
+        res = (char**)malloc(sizeof(char*) * (idx + 2));
+        res[idx + 1] = NULL;
+    }
+    res[idx] = item;
+    return res;
+}
+
+static char**
+create_word_array(const char* cptr)
+{
+    size_t tmplen = strlen(cptr);
+    char* tmp = (char*)malloc(tmplen + 1);
+    char** res = create_word_array_helper(cptr, 0, tmp, tmplen + 1);
+    free(tmp);
+    return res;
+}
+
+static void
+destroy_word_array(char** arr)
+{
+    if (arr) {
+        char** run = arr;
+        while(*run) {
+            free(*run);
+            ++run;
+        }
+        free(arr);
+    }
+}
+
 struct netsnmp_lookup_domain {
     char* application;
-    char* userDomain;
-    char* domain;
+    char** userDomain;
+    char** domain;
     struct netsnmp_lookup_domain* next;
 };
 
@@ -27,8 +67,8 @@ netsnmp_register_default_domain(const char* application, const char* domain)
     }
     if (run && strcmp(run->application, application) == 0) {
       if (run->domain != NULL) {
-	  free (run->domain);
-	  run->domain=NULL;
+          destroy_word_array(run->domain);
+	  run->domain = NULL;
 	  res = 1;
       }
     } else {
@@ -44,7 +84,7 @@ netsnmp_register_default_domain(const char* application, const char* domain)
 	}
     }
     if (domain) {
-	run->domain = strdup(domain);
+        run->domain = create_word_array(domain);
     } else if (run->userDomain == NULL) {
 	if (prev)
 	    prev->next = run->next;
@@ -63,8 +103,8 @@ netsnmp_clear_default_domain(void)
 	struct netsnmp_lookup_domain *tmp = domains;
 	domains = domains->next;
 	free(tmp->application);
-	free(tmp->userDomain);
-	free(tmp->domain);
+        destroy_word_array(tmp->userDomain);
+        destroy_word_array(tmp->domain);
 	free(tmp);
     }
 }
@@ -75,13 +115,17 @@ netsnmp_register_user_domain(const char* token, char* cptr)
     struct netsnmp_lookup_domain *run = domains, *prev = NULL;
     size_t len = strlen(cptr) + 1;
     char* application = (char*)malloc(len);
-    char* domain = (char*)malloc(len);
+    char** domain;
 
     {
-	char* cp = copy_nword(cptr, application, len);
-	cp = copy_nword(cp, domain, len);
-	if (cp)
-	    config_pwarn("Trailing junk found");
+        char* cp = copy_nword(cptr, application, len);
+        if (cp == NULL) {
+            netsnmp_config_error("No domain(s) in registration of "
+                                 "defDomain \"%s\"", application);
+            free(application);
+            return;
+        }
+        domain = create_word_array(cp);
     }
 
     while (run != NULL && strcmp(run->application, application) < 0) {
@@ -92,7 +136,9 @@ netsnmp_register_user_domain(const char* token, char* cptr)
 	if (run->userDomain != NULL) {
 	    config_perror("Default transport already registered for this "
 			  "application");
-	    goto done;
+            destroy_word_array(domain);
+            free(application);
+	    return;
 	}
     } else {
 	run = SNMP_MALLOC_STRUCT(netsnmp_lookup_domain);
@@ -106,9 +152,7 @@ netsnmp_register_user_domain(const char* token, char* cptr)
 	    domains = run;
 	}
     }
-    run->userDomain = strdup(domain);
- done:
-    free(domain);
+    run->userDomain = domain;
     free(application);
 }
 
@@ -119,7 +163,7 @@ netsnmp_clear_user_domain(void)
 
     while (run) {
 	if (run->userDomain != NULL) {
-	    free(run->userDomain);
+            destroy_word_array(run->userDomain);
 	    run->userDomain = NULL;
 	}
 	if (run->domain == NULL) {
@@ -137,33 +181,48 @@ netsnmp_clear_user_domain(void)
     }
 }
 
-const char*
-netsnmp_lookup_default_domain(const char* application)
+const char* const *
+netsnmp_lookup_default_domains(const char* application)
 {
-    const char *res;
+    const char * const * res;
 
     if (application == NULL)
 	res = NULL;
     else {
-	struct netsnmp_lookup_domain *run = domains;
+        struct netsnmp_lookup_domain *run = domains;
 
 	while (run && strcmp(run->application, application) < 0)
 	    run = run->next;
 	if (run && strcmp(run->application, application) == 0)
 	    if (run->userDomain)
-		res = run->userDomain;
+                res = (const char * const *)run->userDomain;
 	    else
-		res = run->domain;
+                res = (const char * const *)run->domain;
 	else
 	    res = NULL;
     }
     DEBUGMSGTL(("defaults",
-		"netsnmp_lookup_default_domain(\"%s\") -> \"%s\"\n",
-		application ? application : "[NIL]",
-		res ? res : "[NIL]"));
+                "netsnmp_lookup_default_domain(\"%s\") ->",
+                application ? application : "[NIL]"));
+    if (res) {
+        const char * const * r = res;
+        while(*r) {
+            DEBUGMSG(("defaults", " \"%s\"", *r));
+            ++r;
+        }
+        DEBUGMSG(("defaults", "\n"));
+    } else
+        DEBUGMSG(("defaults", " \"[NIL]\"\n"));
     return res;
 }
 
+const char*
+netsnmp_lookup_default_domain(const char* application)
+{
+    const char * const * res = netsnmp_lookup_default_domains(application);
+    return (res ? *res : NULL);
+}
+
 struct netsnmp_lookup_target {
     char* application;
     char* domain;
@@ -174,12 +233,21 @@ struct netsnmp_lookup_target {
 
 static struct netsnmp_lookup_target* targets = NULL;
 
+/**
+ * Add an (application, domain, target) triplet to the targets list if target
+ * != NULL. Remove an entry if target == NULL and the userTarget pointer for
+ * the entry found is also NULL. Keep at most one target per (application,
+ * domain) pair.
+ *
+ * @return 1 if an entry for (application, domain) was already present in the
+ *   targets list or 0 if such an entry was not yet present in the targets list.
+ */
 int
 netsnmp_register_default_target(const char* application, const char* domain,
 				const char* target)
 {
     struct netsnmp_lookup_target *run = targets, *prev = NULL;
-    int i, res = 0;
+    int i = 0, res = 0;
     while (run && ((i = strcmp(run->application, application)) < 0 ||
 		   (i == 0 && strcmp(run->domain, domain) < 0))) {
 	prev = run;
@@ -218,6 +286,9 @@ netsnmp_register_default_target(const char* application, const char* domain,
     return res;
 }
 
+/**
+ * Clear the targets list.
+ */
 void
 netsnmp_clear_default_target(void)
 {
@@ -240,11 +311,25 @@ netsnmp_register_user_target(const char* token, char* cptr)
     char* application = (char*)malloc(len);
     char* domain = (char*)malloc(len);
     char* target = (char*)malloc(len);
-    int i;
+    int i = 0;
 
+    if (application == NULL || domain == NULL || target == NULL) {
+        goto done;
+    }
     {
 	char* cp = copy_nword(cptr, application, len);
+        if (cp == NULL) {
+            netsnmp_config_error("No domain and target in registration of "
+                                 "defTarget \"%s\"", application);
+            goto done;
+        }
 	cp = copy_nword(cp, domain, len);
+        if (cp == NULL) {
+            netsnmp_config_error("No target in registration of "
+                                 "defTarget \"%s\" \"%s\"",
+                                 application, domain);
+            goto done;
+        }
 	cp = copy_nword(cp, target, len);
 	if (cp)
 	    config_pwarn("Trailing junk found");
@@ -310,7 +395,7 @@ netsnmp_clear_user_target(void)
 const char*
 netsnmp_lookup_default_target(const char* application, const char* domain)
 {
-    int i;
+    int i = 0;
     struct netsnmp_lookup_target *run = targets;
     const char *res;
 
