@@ -4,21 +4,21 @@
 
 netsnmp_feature_require(cert_util);
 
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
 #endif
-#if HAVE_SYS_SOCKET_H
+#ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif
-#if HAVE_ARPA_INET_H
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
-#if HAVE_NETDB_H
+#ifdef HAVE_NETDB_H
 #include <netdb.h>
 #endif
 #include <errno.h>
@@ -50,17 +50,28 @@ netsnmp_feature_require(cert_util);
 #include <net-snmp/library/system.h>
 #include <net-snmp/library/snmpTLSBaseDomain.h>
 
+#ifndef X509_CHECK_FLAG_MULTI_LABEL_WILDCARDS
+#define X509_CHECK_FLAG_MULTI_LABEL_WILDCARDS 0
+#endif
+
 #define LOGANDDIE(msg) do { snmp_log(LOG_ERR, "%s\n", msg); return 0; } while(0)
 
 int openssl_local_index;
 
 #ifndef HAVE_ERR_GET_ERROR_ALL
-/* A backport of the OpenSSL 1.1.1e ERR_get_error_all() function. */
+/*
+ * A backport of the OpenSSL 3.0 ERR_get_error_all() function. See also
+ * OpenSSL commit b13342e933c5 ("Modernise the ERR functionality further (new
+ * functions and deprecations)").
+ */
 static unsigned long ERR_get_error_all(const char **file, int *line,
                                        const char **func,
                                        const char **data, int *flags)
 {
-    *func = NULL;
+    *func = "(?)";
+#ifdef HAVE_ERR_GET_ERROR_FUNC
+    ERR_get_error_func(func);
+#endif
     return ERR_get_error_line_data(file, line, data, flags);
 }
 #endif
@@ -81,7 +92,7 @@ int verify_callback(int ok, X509_STORE_CTX *ctx) {
     /* things to do: */
 
     X509_NAME_oneline(X509_get_subject_name(thecert), buf, sizeof(buf));
-    fingerprint = netsnmp_openssl_cert_get_fingerprint(thecert, -1);
+    fingerprint = netsnmp_openssl_cert_get_fingerprint(thecert, NS_HASH_SHA1);
     DEBUGMSGTL(("tls_x509:verify", "Cert: %s\n", buf));
     DEBUGMSGTL(("tls_x509:verify", "  fp: %s\n", fingerprint ?
                 fingerprint : "unknown"));
@@ -160,7 +171,7 @@ _netsnmp_tlsbase_verify_remote_fingerprint(X509 *remote_cert,
     char            *fingerprint;
 
     fingerprint =
-        netsnmp_openssl_cert_get_fingerprint(remote_cert, -1);
+        netsnmp_openssl_cert_get_fingerprint(remote_cert, NS_HASH_SHA1);
 
     if (!fingerprint) {
         /* no peer cert */
@@ -177,7 +188,7 @@ _netsnmp_tlsbase_verify_remote_fingerprint(X509 *remote_cert,
 
         if (peer_cert)
             tlsdata->their_fingerprint =
-                netsnmp_openssl_cert_get_fingerprint(peer_cert->ocert, -1);
+                netsnmp_openssl_cert_get_fingerprint(peer_cert->ocert, NS_HASH_SHA1);
     }
 
     if (!tlsdata->their_fingerprint && try_default) {
@@ -189,7 +200,7 @@ _netsnmp_tlsbase_verify_remote_fingerprint(X509 *remote_cert,
 
         if (peer_cert)
             tlsdata->their_fingerprint =
-                netsnmp_openssl_cert_get_fingerprint(peer_cert->ocert, -1);
+                netsnmp_openssl_cert_get_fingerprint(peer_cert->ocert, NS_HASH_SHA1);
     }
     
     if (tlsdata->their_fingerprint) {
@@ -222,8 +233,9 @@ netsnmp_tlsbase_verify_server_cert(SSL *ssl, _netsnmpTLSBaseData *tlsdata) {
     
     netsnmp_assert_or_return(ssl != NULL, SNMPERR_GENERR);
     netsnmp_assert_or_return(tlsdata != NULL, SNMPERR_GENERR);
-    
-    if (NULL == (remote_cert = SSL_get_peer_certificate(ssl))) {
+
+    remote_cert = SSL_get_peer_certificate(ssl);
+    if (!remote_cert) {
         /* no peer cert */
         DEBUGMSGTL(("tls_x509:verify",
                     "remote connection provided no certificate (yet)\n"));
@@ -276,11 +288,9 @@ netsnmp_tlsbase_verify_server_cert(SSL *ssl, _netsnmpTLSBaseData *tlsdata) {
                                             oname->d.ia5);
 
                         /* convert to lowercase for comparisons */
-                        for (j = 0 ;
-                             *check_name && j < sizeof(buf)-1;
-                             ++check_name, ++j ) {
-                            if (isascii(*check_name))
-                                buf[j] = tolower(0xFF & *check_name);
+                        for (j = 0; *check_name && j < sizeof(buf)-1;
+                             ++check_name, ++j) {
+                            buf[j] = tolower(0xFF & *check_name);
                         }
                         if (j < sizeof(buf))
                             buf[j] = '\0';
@@ -355,7 +365,8 @@ netsnmp_tlsbase_verify_client_cert(SSL *ssl, _netsnmpTLSBaseData *tlsdata) {
          above.
        + fingerprint verification happens below.
     */
-    if (NULL == (remote_cert = SSL_get_peer_certificate(ssl))) {
+    remote_cert = SSL_get_peer_certificate(ssl);
+    if (!remote_cert) {
         /* no peer cert */
         DEBUGMSGTL(("tls_x509:verify",
                     "remote connection provided no certificate (yet)\n"));
@@ -450,7 +461,7 @@ _trust_this_cert(SSL_CTX *the_ctx, char *certspec) {
         LOGANDDIE("failed to find requested certificate to trust");
         
     /* Add the certificate to the context */
-    if (netsnmp_cert_trust_ca(the_ctx, trustcert) != SNMPERR_SUCCESS)
+    if (netsnmp_cert_trust(the_ctx, trustcert) != SNMPERR_SUCCESS)
         LOGANDDIE("failed to load trust certificate");
 
     return 1;
@@ -490,7 +501,7 @@ _sslctx_common_setup(SSL_CTX *the_ctx, _netsnmpTLSBaseData *tlsbase) {
                                     NETSNMP_DS_LIB_X509_CRL_FILE);
     if (NULL != crlFile) {
         cert_store = SSL_CTX_get_cert_store(the_ctx);
-        DEBUGMSGTL(("sslctx_client", "loading CRL: %s\n", crlFile));
+        DEBUGMSGTL(("sslctx_common", "loading CRL: %s\n", crlFile));
         if (!cert_store)
             LOGANDDIE("failed to find certificate store");
         if (!(lookup = X509_STORE_add_lookup(cert_store, X509_LOOKUP_file())))
@@ -517,6 +528,8 @@ SSL_CTX *
 sslctx_client_setup(const SSL_METHOD *method, _netsnmpTLSBaseData *tlsbase) {
     netsnmp_cert *id_cert, *peer_cert;
     SSL_CTX      *the_ctx;
+    X509         *ocert;
+    const char   *msg;
 
     /***********************************************************************
      * Set up the client context
@@ -524,7 +537,8 @@ sslctx_client_setup(const SSL_METHOD *method, _netsnmpTLSBaseData *tlsbase) {
     the_ctx = SSL_CTX_new(NETSNMP_REMOVE_CONST(SSL_METHOD *, method));
     if (!the_ctx) {
         snmp_log(LOG_ERR, "ack: %p\n", the_ctx);
-        LOGANDDIE("can't create a new context");
+        msg = "can't create a new context";
+        goto err;
     }
     MAKE_MEM_DEFINED(the_ctx, 256/*sizeof(*the_ctx)*/);
     SSL_CTX_set_read_ahead (the_ctx, 1); /* Required for DTLS */
@@ -544,25 +558,48 @@ sslctx_client_setup(const SSL_METHOD *method, _netsnmpTLSBaseData *tlsbase) {
         id_cert = netsnmp_cert_find(NS_CERT_IDENTITY, NS_CERTKEY_DEFAULT, NULL);
     }
 
-    if (!id_cert)
-        LOGANDDIE ("error finding client identity keys");
+    if (!id_cert) {
+        msg = "error finding client identity keys";
+        goto err;
+    }
 
-    if (!id_cert->key || !id_cert->key->okey)
-        LOGANDDIE("failed to load private key");
+    if (!id_cert->key || !id_cert->key->okey) {
+        msg = "failed to load private key";
+        goto err;
+    }
 
     DEBUGMSGTL(("sslctx_client", "using public key: %s\n",
                 id_cert->info.filename));
     DEBUGMSGTL(("sslctx_client", "using private key: %s\n",
                 id_cert->key->info.filename));
 
-    if (SSL_CTX_use_certificate(the_ctx, id_cert->ocert) <= 0)
-        LOGANDDIE("failed to set the certificate to use");
+    if (SSL_CTX_use_certificate(the_ctx, id_cert->ocert) <= 0) {
+        msg = "failed to set the client certificate to use";
+        goto err;
+    }
 
-    if (SSL_CTX_use_PrivateKey(the_ctx, id_cert->key->okey) <= 0)
-        LOGANDDIE("failed to set the private key to use");
+    if (SSL_CTX_use_PrivateKey(the_ctx, id_cert->key->okey) <= 0) {
+        msg = "failed to set the client private key to use";
+        goto err;
+    }
 
-    if (!SSL_CTX_check_private_key(the_ctx))
-        LOGANDDIE("public and private keys incompatible");
+    if (!SSL_CTX_check_private_key(the_ctx)) {
+        msg = "client public and private keys incompatible";
+        goto err;
+    }
+
+    while (id_cert->issuer_cert) {
+        id_cert = id_cert->issuer_cert;
+        if (id_cert->ocert)
+        {
+            ocert = X509_dup(id_cert->ocert);
+            DEBUGMSGTL(("sslctx_client", "adding cert-chain certificate %p", ocert));
+            if (!ocert || !SSL_CTX_add_extra_chain_cert(the_ctx, ocert)) {
+                msg = "failed to add intermediate client certificate";
+                goto err;
+            }
+        }
+    }
 
     if (tlsbase->their_identity)
         peer_cert = netsnmp_cert_find(NS_CERT_REMOTE_PEER,
@@ -576,22 +613,33 @@ sslctx_client_setup(const SSL_METHOD *method, _netsnmpTLSBaseData *tlsbase) {
                     peer_cert ? peer_cert->info.filename : "none"));
 
         /* Trust the expected certificate */
-        if (netsnmp_cert_trust_ca(the_ctx, peer_cert) != SNMPERR_SUCCESS)
-            LOGANDDIE ("failed to set verify paths");
+        if (netsnmp_cert_trust(the_ctx, peer_cert) != SNMPERR_SUCCESS) {
+            msg = "failed to set verify paths";
+            goto err;
+        }
     }
 
-    /* trust a certificate (possibly a CA) aspecifically passed in */
+    /* trust a certificate (possibly a CA) specifically passed in */
     if (tlsbase->trust_cert) {
-        if (!_trust_this_cert(the_ctx, tlsbase->trust_cert))
+        if (!_trust_this_cert(the_ctx, tlsbase->trust_cert)) {
+            SSL_CTX_free(the_ctx);
             return 0;
+        }
     }
 
     return _sslctx_common_setup(the_ctx, tlsbase);
+
+err:
+    snmp_log(LOG_ERR, "%s\n", msg);
+    SSL_CTX_free(the_ctx);
+    return 0;
 }
 
 SSL_CTX *
 sslctx_server_setup(const SSL_METHOD *method) {
     netsnmp_cert *id_cert;
+    X509         *ocert;
+    const char	 *msg;
 
     /***********************************************************************
      * Set up the server context
@@ -599,30 +647,54 @@ sslctx_server_setup(const SSL_METHOD *method) {
     /* setting up for ssl */
     SSL_CTX *the_ctx = SSL_CTX_new(NETSNMP_REMOVE_CONST(SSL_METHOD *, method));
     if (!the_ctx) {
-        LOGANDDIE("can't create a new context");
+        msg = "can't create a new server context";
+        goto err;
     }
     MAKE_MEM_DEFINED(the_ctx, 256/*sizeof(*the_ctx)*/);
 
     id_cert = netsnmp_cert_find(NS_CERT_IDENTITY, NS_CERTKEY_DEFAULT, NULL);
-    if (!id_cert)
-        LOGANDDIE ("error finding server identity keys");
+    if (!id_cert) {
+        msg = "error finding server identity keys";
+        goto err;
+    }
 
-    if (!id_cert->key || !id_cert->key->okey)
-        LOGANDDIE("failed to load private key");
+    if (!id_cert->key || !id_cert->key->okey) {
+        msg = "failed to load server private key";
+        goto err;
+    }
 
     DEBUGMSGTL(("sslctx_server", "using public key: %s\n",
                 id_cert->info.filename));
     DEBUGMSGTL(("sslctx_server", "using private key: %s\n",
                 id_cert->key->info.filename));
 
-    if (SSL_CTX_use_certificate(the_ctx, id_cert->ocert) <= 0)
-        LOGANDDIE("failed to set the certificate to use");
+    if (SSL_CTX_use_certificate(the_ctx, id_cert->ocert) <= 0) {
+        msg = "failed to set the server certificate to use";
+        goto err;
+    }
 
-    if (SSL_CTX_use_PrivateKey(the_ctx, id_cert->key->okey) <= 0)
-        LOGANDDIE("failed to set the private key to use");
+    if (SSL_CTX_use_PrivateKey(the_ctx, id_cert->key->okey) <= 0) {
+        msg = "failed to set the server private key to use";
+        goto err;
+    }
 
-    if (!SSL_CTX_check_private_key(the_ctx))
-        LOGANDDIE("public and private keys incompatible");
+    if (!SSL_CTX_check_private_key(the_ctx)) {
+        msg ="server public and private keys incompatible";
+        goto err;
+    }
+
+    while (id_cert->issuer_cert) {
+        id_cert = id_cert->issuer_cert;
+        if (id_cert->ocert)
+        {
+            ocert = X509_dup(id_cert->ocert);
+            DEBUGMSGTL(("sslctx_server", "adding cert-chain certificate %p", ocert));
+            if (!ocert || !SSL_CTX_add_extra_chain_cert(the_ctx, ocert)) {
+                msg = "failed to add intermediate server certificate";
+                goto err;
+            }
+        }
+    }
 
     SSL_CTX_set_read_ahead(the_ctx, 1); /* XXX: DTLS only? */
 
@@ -632,7 +704,14 @@ sslctx_server_setup(const SSL_METHOD *method) {
                        SSL_VERIFY_CLIENT_ONCE,
                        &verify_callback);
 
+    SSL_CTX_set_options(the_ctx, SSL_OP_CIPHER_SERVER_PREFERENCE);
+
     return _sslctx_common_setup(the_ctx, NULL);
+    
+err:
+    snmp_log(LOG_ERR, "%s\n", msg);
+    SSL_CTX_free(the_ctx);
+    return NULL;
 }
 
 int
@@ -728,7 +807,7 @@ tls_bootstrap(int majorid, int minorid, void *serverarg, void *clientarg) {
 }
 
 int
-tls_get_verify_info_index() {
+tls_get_verify_info_index(void) {
     return openssl_local_index;
 }
 
@@ -1185,17 +1264,10 @@ void _openssl_log_error(int rc, SSL *con, const char *location) {
         /* if we have a text translation: */
         if (data && (flags & ERR_TXT_STRING)) {
             snmp_log(LOG_ERR, "  Textual Error: %s\n", data);
-            /*
-             * per openssl man page: If it has been allocated by
-             * OPENSSL_malloc(), *flags&ERR_TXT_MALLOCED is true.
-             *
-             * arggh... stupid openssl prototype for ERR_get_error_line_data
-             * wants a const char **, but returns something that we might
-             * need to free??
-             */
-            if (flags & ERR_TXT_MALLOCED)
-                OPENSSL_free(NETSNMP_REMOVE_CONST(void *, data));        }
+        }
     }
-    
+    /* clear openssl error ring buffer */
+    ERR_clear_error();
+
     snmp_log(LOG_ERR, "---- End of OpenSSL Errors ----\n");
 }
